package ap;

import ap.parser.PlainTranslator
import ap.terfor.{Conjunction, Formula}
import ap.proof.{ModelSearchProver, ExhaustiveProver, ConstraintSimplifier, ProofTree}
import ap.proof.goal.Goal

abstract class AbstractFileProver(file : java.io.File, output : Boolean) {

  protected def println(str : => String) : unit = (if (output) Predef.println(str))
  
  val (formula, eliminatedConstants, order) = {
    println("Loading file " + file + " ...")
    val reader = new java.io.FileReader(file)
  
    val input = PlainTranslator(reader)
    reader.close
    input
  }

  protected def findCounterModel : Conjunction = {
    println("Constructing countermodel ...")
    ModelSearchProver(formula, order)
  }
  
  protected def findModel(f : Formula) : Conjunction =
    ModelSearchProver(Conjunction.negate(f, order), order)
  
  protected def constructProofTree(simplifier : ConstraintSimplifier,
                                   timeout : Int) : ProofTree = {
    val initialGoal = Goal(formula, eliminatedConstants, order)
    
    println("Proving ...")
    
    val timeBefore = System.currentTimeMillis
    def stoppingCond(tree : ProofTree) : Boolean =
      (System.currentTimeMillis - timeBefore > timeout) ||
      !tree.closingConstraint.isFalse &&
      // check whether the closing contraint is satisfiable
      !findModel(tree.closingConstraint).isFalse
      
    new ExhaustiveProver(simplifier, stoppingCond _)(initialGoal)
  }

}
