package ap;

import ap.proof.{ConstraintSimplifier, ProofTree}
import ap.terfor.{Conjunction, Quantifier}

object IntelliFileProver {
  
  abstract class Result
  case class Proof(tree : ProofTree, model : Conjunction) extends Result
  case object NoProof extends Result
  case class CounterModel(counterModel : Conjunction) extends Result
  case object NoCounterModel extends Result
  
}

/**
 * A prover that decides, depending on the kind of the problem, whether it
 * should try to construct a proof tree or just search for counterexamples
 */
class IntelliFileProver(file : java.io.File,
                        simplifier : ConstraintSimplifier,
                        // a timeout in milliseconds
                        timeout : Int,
                        output : Boolean)
      extends AbstractFileProver(file, output) {

  lazy val proofTree : ProofTree = constructProofTree(simplifier, timeout)

  lazy val counterModel : Conjunction = findCounterModel

  val result : IntelliFileProver.Result = {
    if ((formula.constants subsetOf eliminatedConstants) &&
        (Conjunction.collectQuantifiers(formula) subsetOf Set(Quantifier.ALL))) {
      // try to find a countermodel
      if (counterModel.isFalse)
        IntelliFileProver.NoCounterModel
      else
        IntelliFileProver.CounterModel(counterModel)
    } else {
      // try to construct a proof
      val model = findModel(proofTree.closingConstraint)
      if (model.isFalse)
        IntelliFileProver.NoProof
      else
        IntelliFileProver.Proof(proofTree, model)
    }
  }
  
}
