package ap.proof;

import ap.terfor.Quantifier
import ap.proof.goal.{PrioritisedTask, Goal}
import ap.terfor.{Formula, Conjunction, ConstantTerm, TermOrder}
import ap.terfor.substitutions.Substitution

/**
 * Proof tree factory in which updating a goal will recursive apply further
 * rules, until some stopping condition holds
 */
class IteratingProofTreeFactory(stoppingCond : (Goal) => Boolean,
                                simplifier : ConstraintSimplifier)
      extends SimpleProofTreeFactory(true, simplifier) {

  override def updateGoal(updatedFacts : Conjunction,
                          updatedElimConstants : Set[ConstantTerm],
                          updatedOrder : TermOrder,
                          newTasks : Iterable[PrioritisedTask],
                          goal : Goal) : ProofTree = {
    val updatedTasks = goal.tasks.removeFirst ++ newTasks
    val newGoal = Goal(updatedFacts,
                       updatedTasks,
                       goal.age + 1,
                       updatedElimConstants,
                       updatedOrder)
    if (!newGoal.stepPossible || stoppingCond(newGoal))
      newGoal
    else
      newGoal step this
  }

  
}
