package ap.proof;

import ap.terfor.{Formula, TermOrder}
import ap.terfor.substitutions.{Substitution, ComposeSubsts}
import ap.util.Debug

object SubstTree {
  
  private val AC = Debug.AC_PROOF_TREE

  def apply(subtree : ProofTree, subst : Substitution,
            order : TermOrder, subtreeOrder : TermOrder,
            simplifier : ConstraintSimplifier)
                                              : ProofTree = subtree match {
    case subtree : SubstTree =>
      SubstTree(subtree.subtree,
                ComposeSubsts(subst sortBy subtree.subtreeOrder,
                              subtree.subst,
                              subtree.subtreeOrder),
                order, subtree.subtreeOrder, simplifier)
    case subtree : AndTree =>
      AndTree(for (t <- subtree.subtrees)
              yield SubstTree(t, subst, order, subtreeOrder, simplifier),
              order, simplifier)
    case subtree : WeakenTree =>
      WeakenTree(SubstTree(subtree.subtree, subst,
                           order, subtreeOrder, simplifier),
                 order sort subst(subtree.disjunct),
                 order)
    case _ => new SubstTree(subtree, subst, order, subtreeOrder, simplifier)
  }
  
}

/**
 * <code>ProofTreeOneChild</code> that applies a substitution to the closing
 * constraint of its <code>subtree</code>
 */
class SubstTree private (val subtree : ProofTree,
                         val subst : Substitution,
                         protected val order : TermOrder,
                         protected val subtreeOrder : TermOrder,
                         simplifier : ConstraintSimplifier)
      extends ProofTreeOneChild {

  //////////////////////////////////////////////////////////////////////////////
  Debug.assertCtor(SubstTree.AC,
                   (subst isSortedBy subtreeOrder) &&
                   (subtree match {
                     // nested substitutions or weakening are supposed to be
                     // contracted
                     case _ : SubstTree | _ : WeakenTree | _ : AndTree => false
                     case _ => true
                    }))
  //////////////////////////////////////////////////////////////////////////////

  lazy val closingConstraint : Formula =
    (order sort subst(subtree.closingConstraint))

  lazy val stepMeaningful : Boolean = subtree.stepMeaningful

  def update(newSubtree : ProofTree) : ProofTree =
    SubstTree(newSubtree, subst, order, subtreeOrder, simplifier)

  //////////////////////////////////////////////////////////////////////////////

  override def toString : String = {
    "SubstTree (" + subst + ") (" + closingConstraint + ") {\n" + subtree + "\n}"
  }
}
