package ap.proof;

import ap.util.{Debug, Logic, APTestCase, PlainRange}
import ap.terfor.{TestGenConjunctions, Conjunction, Quantifier}
import ap.proof.goal.Goal

class TestRandomProving(n : String) extends APTestCase(n) {

  private val tg = new TestGenConjunctions
  
  import tg.{randomEqConj, randomConj, to}
  
  def runTest = {
    n match {
      // proving with only equations
      case "testEqFormulas1" => testProveFormulas(6, 100,
                                                  randomEqConj(_, 5, 8),
                                                  ConstraintSimplifier.NO_SIMPLIFIER)
      case "testEqFormulas2" => testProveFormulas(5, 60,
                                                  randomEqConj(_, 4, 6),
                                                  ConstraintSimplifier.ALL_SIMPLIFIER)
      // proving with equations and inequalities
      case "testFormulas1" =>   testProveFormulas(6, 100,
                                                  randomConj(_, 5, 8),
                                                  ConstraintSimplifier.NO_SIMPLIFIER)
      case "testFormulas2" =>   testProveFormulas(5, 60,
                                                  randomConj(_, 4, 6),
                                                  ConstraintSimplifier.ALL_SIMPLIFIER)
    }
  }

  private def testProveFormulas(maxSize : Int, iterations : Int,
                                forGen : (Int) => Conjunction,
                                simplifier : ConstraintSimplifier) = {
    val prover = new ExhaustiveProver(simplifier)
    
    for (sizeFactor <- PlainRange(0, maxSize); _ <- PlainRange(iterations)) {
      var formula = forGen(sizeFactor)

      // ensure that the formula does not contain free variables
      while (!formula.variables.isEmpty)
        formula = Conjunction.quantify(Array(Quantifier.ALL), formula, to)
      
/*            println
            println(formula) */

      val goal = Goal(formula, to)

      val tree = prover(goal)
      tree.closingConstraint

/*      println
    println("After many steps:")
println(tree) */

      TestProofTree.assertNormalisedTree(tree)
    }
  }
    
}
