package ap.util;

object PeekIterator {
  def apply[T](it : Iterator[T]) : PeekIterator[T] = it match {
    case it : PeekIterator[T] => it
    case _ => new PeekIteratorTrafo[T](it)
  }
  
  def single[T](value : T) : PeekIterator[T] = new PeekIterator[T] {
    private var hasnext = true
    def hasNext = hasnext
    def next = {
      hasnext = false
      value
    }
    def peekNext = value
  }
}

class PeekIteratorTrafo[T](delegate : Iterator[T]) extends PeekIterator[T] {
  
  private var nextVal : Option[T] = None
  
  def hasNext = nextVal.isDefined || delegate.hasNext
  
  def next : T = nextVal match {
    case None => delegate.next
    case Some(v) => { nextVal = None; v }
  }
  
  def peekNext : T = nextVal match {
    case None => { val v = delegate.next; nextVal = Some(v); v }
    case Some(v) => v
  }
}

/**
 * Extension of the iterator trait where the next element can be peeked without
 * popping it
 */
trait PeekIterator[+T] extends Iterator[T] {
  def peekNext : T
  
  def dropAll : Unit = {
    while (hasNext) next
  }
}
